//
// Author: Benjamin Langmann <b.langmann@gmx.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef LMJPEG_IO_H
#define LMJPEG_IO_H

#ifndef VFW_WIN
#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#endif

#define LMJPEG_IO_FILEHEADER  0xf0f0f0f0
#define LMJPEG_IO_FRAMEHEADER 0xf1f0

#define LMJPEG_IO_FRAMEHEADERMASK 0xfff8
#define LMJPEG_IO_PFRAME 0x0001
#define LMJPEG_IO_KEYFRAME 0x0002
#define LMJPEG_IO_NFRAME 0x0004

/** The file header struct contains information about how to encode and decode the video. It is saved once per video.*/
typedef struct {
	int32_t marker;		/**< identification*/
	int32_t width;		/**< width of the video*/
	int32_t height;		/**< height of the video*/
	int32_t topdown;	/**< identifies if each image is stored top-down or buttom-up*/
	int32_t time_base_num;	/**< frame rate numerator*/
	int32_t time_base_den;	/**< frame rate denumerator*/
	int32_t keyframe_intervall;/**< keyframe interval*/
	int32_t dyn_keyframes;	/**< set keyframes dynamically*/
	int32_t quality_mode;	/**< quality methode used*/
	int32_t quality_num;	/**< quality parameter one*/
	int32_t quality_diff;	/**< quality parameter two*/
	int32_t jpeg_quality;	/**< quantization parameter used:
					0-99 : scale linear from QM_0 to QM_99
					100  : lossless mode
					101 <= x <= 131 : use quantizer x-100
					132 : QM stored in myqm
				*/
	int32_t pix_format;	/**< pixel format used*/
	int32_t compression;	/**< entropy encoding used*/
	uint8_t myqm[64];	/**< quantization matrix used*/
	uint8_t ratings[64];	/**< rating matrix used*/
} lmjpeg_fileheader;

/** The CODEC struct contains run time information for the codec.*/
typedef struct {
	lmjpeg_fileheader *config; /**< pointer to the file header struct*/
	int save;		/**< internal use*/
	uint16_t frame_type;
	char filename[128];	/**< output filename*/
	int want_topdown;	/**< should output be top-down or buttom-up*/
	int in_out_pix_format;	/**< output pixel format*/

// private:
	int frame_num;		/**< number of current frame*/
	int keyframe_cnt;	/**< number of frames since the last keyframe*/
	int16_t **last_frame; 	/**< pointer to last frame*/
	FILE *lmjpeg_file;	/**< output file handle*/
	int written_frame_bytes;/**< frame bytes written*/
	int last_frame_num;	/**< number of frame last written*/
	int file_size;		/**< file size so far*/

	int field_widths_int[2]; /**< 2,3 field equal width*/
	int field_heights_int[2];
} CODEC;

/** The frame header struct contains information about a frame and is used before every frame.*/
typedef struct {
	uint16_t marker;	/**< identification and frame type*/
	uint16_t frame_num;	/**< number of current frame*/
	uint32_t size;		/**< frame size in bytes*/
} lmjpeg_frameheader;

int lmjpeg_compress(CODEC *codec, uint16_t *data, int size);
int lmjpeg_decompress(CODEC *codec, uint8_t *data, int size, uint16_t *to, int tosize);

int rle_encode(uint8_t *data, int size, uint8_t *to);
int rle_decode(uint8_t *data, int size, uint8_t *to, int tosize);
int rle_encode_12(uint16_t *data, int size, uint16_t *to);
int rle_decode_12(uint16_t *data, int size, uint16_t *to, int tosize);

int zlib_encode(uint8_t *data, int size, uint8_t *to);
int zlib_decode(uint8_t *data, int size, uint8_t *to, int tosize);

CODEC *setup_codec();
void end_codec(CODEC *co);
int check_config(lmjpeg_fileheader *config);
void lmjpeg_calc_QM(CODEC *codec);


#ifndef VFW_WIN
// ------------------------------------------------
// functions and structs needed to write avi files
// ------------------------------------------------

int lmjpeg_writer_init(CODEC *codec);
int lmjpeg_reader_init(CODEC *codec);

int lmjpeg_write(CODEC *codec, int framenum, uint8_t *img, int size);
int lmjpeg_read(CODEC *codec, lmjpeg_frameheader *fheader, uint8_t *img);
	
void lmjpeg_writer_close(CODEC *codec);
void lmjpeg_reader_close(CODEC *codec);

int lmjpeg_read_int(FILE *file);
int lmjpeg_read_data(FILE *file, void *data, int size);

// from MS vfw.h
typedef uint32_t DWORD;
typedef uint16_t WORD;
typedef int32_t LONG;
/*
** Main AVI File Header
*/	
		
/* flags for use in <dwFlags> in AVIFileHdr */
#define AVIF_HASINDEX		0x00000010	// Index at end of file?
#define AVIF_MUSTUSEINDEX	0x00000020
#define AVIF_ISINTERLEAVED	0x00000100
#define AVIF_WASCAPTUREFILE	0x00010000
#define AVIF_COPYRIGHTED	0x00020000

/* The AVI File Header LIST chunk should be padded to this size */
#define AVI_HEADERSIZE  2048                    // size of AVI header list

typedef struct {
	DWORD		dwMicroSecPerFrame;	// frame display rate (or 0L)
	DWORD		dwMaxBytesPerSec;	// max. transfer rate
	DWORD		dwPaddingGranularity;	// pad to multiples of this
                                                // size; normally 2K.
	DWORD		dwFlags;		// the ever-present flags
	DWORD		dwTotalFrames;		// # frames in file
	DWORD		dwInitialFrames;
	DWORD		dwStreams;
	DWORD		dwSuggestedBufferSize;

	DWORD		dwWidth;
	DWORD		dwHeight;

	DWORD		dwReserved[4];
} lmjpeg_MainAVIHeader;


typedef struct {
	DWORD		fccType;
	DWORD		fccHandler;
	DWORD		dwFlags;	/* Contains AVITF_* flags */
	WORD		wPriority;
	WORD		wLanguage;
	DWORD		dwInitialFrames;
	DWORD		dwScale;	
	DWORD		dwRate;	/* dwRate / dwScale == samples/second */
	DWORD		dwStart;
	DWORD		dwLength; /* In units above... */
	DWORD		dwSuggestedBufferSize;
	DWORD		dwQuality;
	DWORD		dwSampleSize;
	DWORD		rcFrame[2];
} lmjpeg_AVIStreamHeader;

/* Flags for index */
#define AVIIF_LIST          0x00000001L // chunk is a 'LIST'
#define AVIIF_KEYFRAME      0x00000010L // this frame is a key frame.
#define AVIIF_FIRSTPART     0x00000020L // this frame is the start of a partial frame.
#define AVIIF_LASTPART      0x00000040L // this frame is the end of a partial frame.
#define AVIIF_MIDPART       (AVIIF_LASTPART|AVIIF_FIRSTPART)

#define AVIIF_NOTIME	    0x00000100L // this frame doesn't take any time
#define AVIIF_COMPUSE       0x0FFF0000L // these bits are for compressor use

typedef struct {
	DWORD		ckid;
	DWORD		dwFlags;
	DWORD		dwChunkOffset;		// Position of chunk
	DWORD		dwChunkLength;		// Length of chunk
} lmjpeg_AVIINDEXENTRY;

typedef struct tagBITMAPINFOHEADER {
		DWORD      biSize;
        LONG       biWidth;
        LONG       biHeight;
        WORD       biPlanes;
        WORD       biBitCount;
        DWORD      biCompression;
        DWORD      biSizeImage;
        LONG       biXPelsPerMeter;
        LONG       biYPelsPerMeter;
        DWORD      biClrUsed;
        DWORD      biClrImportant;
} lmjpeg_BITMAPINFOHEADER;

#endif

#endif
